/*
AUTOR: MICROSIDE TECHNOLOGY S.A. DE C.V.
FECHA: ABRIL 2022
URLs:
https://microside.com/
https://microside.com/xide
https://microside.com/docs/x-node-2g-lte-gnss-sim7000g/

---------------------------------------------------------------------------
Este ejemplo consiste en enviar datos de temperatura y humedad a través
de la red celular del módulo XC02.
---------------------------------------------------------------------------
*/

#include <Wire.h>
#include "SHTSensor.h"
#define SHT40 SHTSensor::SHT4X
#define XC02 Serial2.println
SHTSensor sht40(SHT40);

/////////////////////////////////////////////////////////////
////////////////////   XC02 CONFIG   ////////////////////////
/////////////////////////////////////////////////////////////

#define CMD1 "AT"
#define CMD2 "AT+IPR=115200"
#define CMD3 "AT+CNMP=2"
#define CMD4 "AT+COPS=0"
#define CMD5 "AT+CFUN=0"
#define CMD6 "AT+CGDCONT=1,\"IP\","
#define CMD7 "AT+CFUN=1"
#define CMD8 "AT+CLTS=1"
#define CMD9 "AT+CNACT=1"

/////////////////////////////////////////////////////////////
////////////////////    HTTPS CONFIG   //////////////////////
/////////////////////////////////////////////////////////////

#define CHT1 "AT+CSSLCFG=\"sslversion\",1,3"
#define CHT2 "AT+SHSSL=1,\"\""
#define CHT3 "AT+SHCONF=\"URL\","
#define CHT4 "AT+SHCONF=\"BODYLEN\",1024"
#define CHT5 "AT+SHCONF=\"HEADERLEN\",350"
#define CHT6 "AT+SHCONN"
#define CHT7 "AT+SHREQ=\"/external/api/batch/update?token="

/////////////////////////////////////////////////////////////
////////////////////    CRITICAL DATA   /////////////////////
/////////////////////////////////////////////////////////////

#define APN "\"mobiledata\"" //APN para SIM multicarrier (898)
//#define APN "\"linksnet\"" //APN para SIM que no es multicarrier (894)
#define BLYNK_CLOUD "\"https://ny3.blynk.cloud:443\""
#define BLYNK_AUTH_TOKEN "YXEOn_vj-XH_imVzw-XEr9P0E93LTk0v"  //modifícalo de acuerdo con tu dispositivo en Blynk.

/////////////////////////////////////////////////////////////
////////////////////    HELPFUL DATA   //////////////////////
/////////////////////////////////////////////////////////////

#define POWERKEY 26
#define RESET 27
#define T 120000
String Data;
float HUM, TEMP;
String inputString = "";
bool stringComplete = false;
bool Control = false;
#define LED 12

/////////////////////////////////////////////////////////////
////////////////////     FUNCTIONS     //////////////////////
/////////////////////////////////////////////////////////////

//---------------------------------------------------------//
// En esta función se configura la conexión automática del //
// módulo, usando la configuración del APN y comandos AT.  //
//---------------------------------------------------------//

void XC02_begin()
{
  XC02(CMD1); XC01_DATA_read("OK\r\n");
  XC02(CMD2); XC01_DATA_read("OK\r\n");
  XC02(CMD3); XC01_DATA_read("OK\r\n");
  XC02(CMD4);  XC01_DATA_read("OK\r\n");
  XC02(CMD5);  XC01_DATA_read("OK\r\n");
  Data = String(CMD6) + String(APN);
  XC02(Data);  XC01_DATA_read("OK\r\n");
  XC02(CMD7);  XC01_DATA_read("SMS Ready\r\n"); delay(1500);
  XC02(CMD8);  XC01_DATA_read("OK\r\n");
  XC02(CMD9);  XC01_DATA_read("+APP PDP: ACTIVE\r\n");
}

//---------------------------------------------------------//
// En esta función se configura la conexión https a Blynk, //
// usando el token generado desde el dispositivo.          //
//---------------------------------------------------------//

void XC02_BLYNK_begin()
{
  XC02(CHT1); XC01_DATA_read("OK\r\n");
  XC02(CHT2); XC01_DATA_read("OK\r\n");
  Data = String(CHT3) + String(BLYNK_CLOUD);
  XC02(Data);  XC01_DATA_read("OK\r\n");
  XC02(CHT4); XC01_DATA_read("OK\r\n");
  XC02(CHT5); XC01_DATA_read("OK\r\n");
  XC02(CHT6); XC01_DATA_read("OK\r\n");
}

//---------------------------------------------------------//
//  Esta función envía los datos obtenidos a blynk.cloud.  //
//  Se debe tener en cuenta los pines virtuales para       //
//  poder hacer la correcta interpretación de los datos    //
//  que se van a visualizar en el dashboard de Blynk.      //
//---------------------------------------------------------//

void XC02_BLYNK_send(float A, float B)
{
  Data = String(CHT7) + String(BLYNK_AUTH_TOKEN) + "&v0=" + String(A) + "&v1=" + String(B) + "\",1";
  XC02(Data);  XC01_DATA_read("+SHREQ: \"GET\",200,0\r\n");
}

//---------------------------------------------------------//
// Esta función se comunica con el sensor para solicitar   //
// los valores de temperatura y humedad y posteriormente   //
// guardarlos en las variables TEMP y HUM respectivamente.  //
//---------------------------------------------------------//

void ReadSensor(void)
{
  sht40.readSample();
  HUM = sht40.getHumidity();
  TEMP = sht40.getTemperature();
}

//---------------------------------------------------------//
// Esta función identifica la respuesta del módulo XC02,   //
// antes de enviar el siguiente comando, mientras ejecuta  //
// un timer para esperar la respuesta.                     //
//---------------------------------------------------------//

void XC01_DATA_read(String REQ)
{
  digitalWrite(LED, LOW);
  stringComplete = false;
  Control = false;
  int timer0 = millis();
  inputString = "";
  char inChar;
  
  while (!stringComplete)
  {
    char inChar;
    while (Serial2.available())
    {
      inChar = (char)Serial2.read();
      inputString += inChar;
      if (inputString == REQ) {
        digitalWrite(LED, HIGH);
        Serial.print(inputString);
        stringComplete = true;
      }
      if (inChar == '\n') {
        inputString = "";
      }
    }
    if (T == (millis() - timer0))
    {
      Serial.println("ERROR... ¡PLEASE CHECK YOUR DEVICES OR NETWORK DATA!");
      Control = true;
      break;
    }
  }
}


void setup() {
  
  //Se configuran las operaciones de XC01.//
  
  Serial.begin(115200);
  Serial2.begin(115200);
  pinMode(LED, OUTPUT);
  pinMode(POWERKEY, OUTPUT);
  pinMode(RESET, OUTPUT);
  /////////////////////////
  
  //Se envían las señales de inicio para resetear y/o encender el modem.//
  
  digitalWrite(LED, LOW);
  digitalWrite(RESET, HIGH);
  delay(500);
  digitalWrite(RESET, LOW);
  digitalWrite(POWERKEY, HIGH);
  delay(150);
  digitalWrite(POWERKEY, LOW);
  //////////////////////////

  //Se espera la respuesta de XC02.//
  
  XC01_DATA_read("SMS Ready\r\n");
  delay(1500);
  /////////////////////////

  //Se inicializa la comunicación con el sensor.//
  
  Wire.begin();
  sht40.init();
  /////////////////////////

  //Se realiza la configuración y se establece la conexión.//
  
  XC02_begin();
  XC02_BLYNK_begin();
  /////////////////////////
}

//Bucle infinito lectura-envio.//

void loop() {
  ReadSensor();
  XC02_BLYNK_send(HUM, TEMP);
  delay(1500);
}
